@@ -126,7 +126,9 @@ class Table(metaclass=MetaTable):
     type = lambda x: None
     schema = None
     name = None
+    id = 'id'
     fields = dict()
+    joins = dict()
     get_db = lambda: None
     put_db = lambda db: None
     @classmethod
@@ -428,14 +430,21 @@ class Table(metaclass=MetaTable):
         #return cls.type(**params)
     @classmethod
     def get(cls, id):
+        row = Row()
+        row.offset(cls.name, cls)
+        join = Join(cls, row)
         try:
             db = cls.get_db()
             cursor = db.cursor()
-            cursor.execute(f"""SELECT {cls.select()}
+            cursor.execute(f"""SELECT {select(cls.select(), join.select())}
                              FROM {cls}
+                             {join}
                              WHERE {cls('id')}=%s""",
                              (id,))
-            return cls.create(cursor.fetchone())
+            row.data(cursor.fetchone())
+            result = cls.create(row(cls.name))
+            join.set(result)
+            return result
         except Exception as error:
             raise error
         finally:
@@ -447,21 +456,29 @@ class Table(metaclass=MetaTable):
         filter = cls.where(filter)
         search = cls.where(search, separator='OR')
 
+        row = Row()
+        row.offset(cls.name, cls)
+        join = Join(cls, row)
+        row.offset('total')
+
         result = []
 
         try:
             db = cls.get_db()
             cursor = db.cursor()
             cursor.execute(*debug(f"""SELECT
-                           {cls.select()}
+                           {select(cls.select(), join.select())}
                            FROM {cls}
+                           {join}
                            WHERE ({filter.fields()}) AND ({search.fields()})
                            ORDER BY {cls.order('id', 'desc', order)}""",
                            filter.values()+search.values()))
             log.debug(color.cyan('Total fetched %s'))
             while True:
                 try:
-                    item = cls.create(cursor.fetchone())
+                    row.data(cursor.fetchone())
+                    item = cls.create(row(cls.name))
+                    join.set(item)
                     result.append(item)
                 except TypeError:
                     break
@@ -481,7 +498,8 @@ class Table(metaclass=MetaTable):
         offset = (page-1)*limit
 
         row = Row()
-        row.offset('table', cls.offset())
+        row.offset(cls.name, cls)
+        join = Join(cls, row)
         row.offset('total')
 
         result = Result()
@@ -490,9 +508,10 @@ class Table(metaclass=MetaTable):
             db = cls.get_db()
             cursor = db.cursor()
             cursor.execute(*debug(f"""SELECT
-                           {cls.select()},
+                           {select(cls.select(), join.select())},
                            COUNT(*) OVER()
                            FROM {cls}
+                           {join}
                            WHERE ({filter.fields()}) AND ({search.fields()})
                            ORDER BY {cls.order('id', 'desc', order)}
                            LIMIT %s OFFSET %s""",
@@ -503,7 +522,8 @@ class Table(metaclass=MetaTable):
                     if result.total is None:
                         result.total = row('total')
                         print('Total', result.total)
-                    item = cls.create(row('table'))
+                    item = cls.create(row(cls.name))
+                    join.set(item)
                     result.add(item)
                 except TypeError:
                     break
@@ -522,15 +542,26 @@ class Table(metaclass=MetaTable):
     def save(cls, id, data, filter={}):
         filter = cls.where(filter)
         update = cls.update(data)
+        row = Row()
+        row.offset(cls.name, cls)
+        join = Join(cls, row)
         try:
             db = cls.get_db()
             cursor = db.cursor()
-            cursor.execute(*debug(f"""UPDATE {cls}
-                                    SET {update.fields()}
-                                    WHERE {cls('id')}=%s AND {filter.fields()}
-                                    RETURNING {cls.select()}""",
+            cursor.execute(*debug(f"""WITH {cls.name} AS (
+                                        UPDATE {cls}
+                                        SET {update.fields()}
+                                        WHERE {cls('id')}=%s AND {filter.fields()}
+                                        RETURNING {cls.select()}
+                                    )
+                                    SELECT {select(f'{cls.name}.*', join.select())}
+                                    FROM {cls.name}
+                                    {join}
+                                    """,
                                 update.values(id)+filter.values()))
-            result = cls.create(cursor.fetchone())
+            row.data(cursor.fetchone())
+            result = cls.create(row(cls.name))
+            join.set(result)
         except Exception as error:
             raise error
         finally:
@@ -541,16 +572,28 @@ class Table(metaclass=MetaTable):
 
     @classmethod
     def add(cls, data):
+        row = Row()
+        row.offset(cls.name, cls)
+        join = Join(cls, row)
         insert = cls.insert(data)
         try:
             db = cls.get_db()
             cursor = db.cursor()
-            cursor.execute(*debug(f"""INSERT INTO {cls}
-                                    ({insert.fields()})
-                                    VALUES ({insert.fields('%s')})
-                                    RETURNING {cls.select()}""",
+            cursor.execute(*debug(f"""WITH {cls.name} AS (
+                                        INSERT INTO {cls}
+                                        ({insert.fields()})
+                                        VALUES ({insert.fields('%s')})
+                                        RETURNING {cls.select()}
+                                    )
+                                    SELECT {select(f'{cls.name}.*', join.select())}
+                                    FROM {cls.name}
+                                    {join}
+                                    """,
                                 insert.values()))
-            result = cls.create(cursor.fetchone())
+
+            row.data(cursor.fetchone())
+            result = cls.create(row(cls.name))
+            join.set(result)
         except Exception as error:
             raise error
         finally:
@@ -599,6 +642,35 @@ class Row:
     def __call__(self, name):
         return self.get(name)
 
+class Join():
+    def __init__(self, table, row):
+        self.table = table
+        self.row = row
+        if self.table.joins:
+            for join in self.table.joins.values():
+                self.row.offset(join['table'].name, join['table'])
+    def select(self):
+        result = ''
+        if self.table.joins:
+            result = ','.join([join['table'].select() for join in self.table.joins.values()])
+        return result
+
+    def clause(self, name):
+        return f"LEFT JOIN {self.table.joins[name]['table']} ON \
+{self.table.joins[name]['table'](self.table.joins[name]['table'].id)}={self.table(self.table.joins[name]['field'])}"
+
+    def __str__(self):
+        if self.table.joins:
+            return '\n'.join([self.clause(join) for join in self.table.joins.keys()])
+        return ''
+
+    def set(self, item):
+        if self.table.joins:
+            for name, join in self.table.joins.items():
+                setattr(item, name, join['table'].create(self.row(join['table'].name)))
+
+def select(*args):
+    return ','.join([item for item in args if str(item).strip()!=''])
 
 class Result():
     def __init__(self, total=None):
@@ -624,6 +696,8 @@ def debug(query, params):
     query_debug = query_debug.replace('WHERE', '\n'+color.green('WHERE'))
     query_debug = query_debug.replace(' AND ', '\n    '+color.yellow('AND')+' ')
     query_debug = query_debug.replace(' OR ', '\n    '+color.red('OR')+' ')
+    query_debug = query_debug.replace(' ON ', ' '+color.cyan('ON')+' ')
+    query_debug = query_debug.replace('WITH ', color.cyan('WITH')+' ')
     query_debug = query_debug.replace('FROM', '\n'+color.green('FROM'))
     query_debug = query_debug.replace('ORDER', '\n'+color.red('ORDER'))
     query_debug = query_debug.replace('LIMIT', '\n'+color.yellow('LIMIT'))
